import { __rest } from 'tslib';
import { Input, ChangeDetectionStrategy, ChangeDetectorRef, Component, Optional, SkipSelf, ViewEncapsulation, Directive, Host, TemplateRef, ViewContainerRef, NgModule } from '@angular/core';
import { Observable, Subject, ReplaySubject } from 'rxjs';
import { filter, startWith, takeUntil } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} obj
 * @return {?}
 */
function isSubscribableOrPromise(obj) {
    return obj instanceof Observable || obj instanceof Subject || obj instanceof Promise;
}
/**
 * @param {?} input
 * @return {?}
 */
function parseKeys(input) {
    return (Array.isArray(input) ? input : input.split(/\s+/)).filter((/**
     * @param {?} key
     * @return {?}
     */
    key => key));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template T
 */
class AbstractContextConsumer {
    /**
     * @param {?} provider
     * @param {?} target
     */
    constructor(provider, target) {
        this.provider = provider;
        this.target = target;
        this.destroy$ = new Subject();
        this._contextMap = {};
        this._consume = '';
        this.consumed = new Map();
    }
    /**
     * @param {?} map
     * @return {?}
     */
    set contextMap(map) {
        this._contextMap = map || {};
    }
    /**
     * @return {?}
     */
    get contextMap() {
        return this._contextMap;
    }
    /**
     * @param {?} consume
     * @return {?}
     */
    set consume(consume) {
        this._consume = consume || '';
    }
    /**
     * @return {?}
     */
    get consume() {
        return this._consume;
    }
    /**
     * @return {?}
     */
    get component() {
        return this.target['_view'].component;
    }
    /**
     * @protected
     * @return {?}
     */
    init() {
        /** @type {?} */
        const consumed = parseKeys(this.consume);
        this.provider.reset$
            .pipe(takeUntil(this.destroy$))
            .subscribe((/**
         * @return {?}
         */
        () => this.ngOnChanges()));
        if (this.provider.provide.length)
            this.provider.change$
                .pipe(takeUntil(this.destroy$), startWith(...Array.from(this.provider.provided.keys())), filter((/**
             * @param {?} key
             * @return {?}
             */
            key => !!key)))
                .subscribe((/**
             * @param {?} providerKey
             * @return {?}
             */
            providerKey => this.syncProperties(consumed, providerKey)));
    }
    /**
     * @protected
     * @return {?}
     */
    reset() {
        this.consumed.forEach((/**
         * @param {?} value
         * @param {?} key
         * @return {?}
         */
        (value, key) => {
            this.component[key] = value;
        }));
        this.consumed.clear();
    }
    /**
     * @protected
     * @param {?} consumed
     * @param {?} providerKey
     * @return {?}
     */
    syncProperties(consumed, providerKey) {
        /** @type {?} */
        let key = this.provider.contextMap[providerKey] || providerKey;
        key = this.contextMap[key] || key;
        if (consumed.length && consumed.indexOf(key) < 0)
            return;
        if (!this.consumed.has(key))
            this.consumed.set(key, this.component[key]);
        this.component[key] = this.provider.component[providerKey];
        this.target.markForCheck();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        if (this.initialized) {
            this.destroy$.next();
            this.reset();
            if (this.target && this.provider)
                this.init();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy$.next();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.initialized = true;
        this.ngOnChanges();
    }
}
AbstractContextConsumer.propDecorators = {
    contextMap: [{ type: Input }],
    consume: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ContextProviderComponent {
    /**
     * @param {?} source
     */
    constructor(source) {
        this.source = source;
        this.initialized = false;
        this._contextMap = {};
        this._provide = '';
        this.provided = new Map();
        this.change$ = new ReplaySubject(1);
        this.reset$ = new Subject();
    }
    /**
     * @param {?} map
     * @return {?}
     */
    set contextMap(map) {
        this._contextMap = map || {};
    }
    /**
     * @return {?}
     */
    get contextMap() {
        return this._contextMap;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set provide(value) {
        this._provide = value || '';
    }
    /**
     * @return {?}
     */
    get provide() {
        return this._provide;
    }
    /**
     * @return {?}
     */
    get component() {
        return this.source['_view'].component;
    }
    /**
     * @private
     * @return {?}
     */
    init() {
        setTimeout((/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const THIS = this;
            /** @type {?} */
            const context = new Map();
            /** @type {?} */
            const provided = parseKeys(this.provide).filter((/**
             * @param {?} key
             * @return {?}
             */
            key => key && key in this.component));
            provided.forEach((/**
             * @param {?} key
             * @return {?}
             */
            key => {
                if (isSubscribableOrPromise(this.component[key])) {
                    this.change$.next(key);
                    return;
                }
                /** @type {?} */
                const propertyDescriptor = Object.getOwnPropertyDescriptor(this.component, key) ||
                    Object.getOwnPropertyDescriptor(((/** @type {?} */ (this.component))).__proto__, key);
                this.provided.set(key, propertyDescriptor);
                const { value, writable, get: getter, set: setter } = propertyDescriptor, prop = __rest(propertyDescriptor, ["value", "writable", "get", "set"]);
                Object.defineProperty(this.component, key, Object.assign({}, prop, { get: getter
                        ? (/**
                         * @return {?}
                         */
                        function () {
                            return getter.call(this);
                        })
                        : (/**
                         * @return {?}
                         */
                        function () {
                            return context.get(key);
                        }), set: setter
                        ? (/**
                         * @return {?}
                         */
                        function () {
                            setter.apply(this, arguments);
                            THIS.change$.next(key);
                        })
                        : (/**
                         * @param {?} newValue
                         * @return {?}
                         */
                        function (newValue) {
                            context.set(key, newValue);
                            THIS.change$.next(key);
                        }) }));
                this.component[key] = value || this.component[key];
            }));
        }), 0);
    }
    /**
     * @private
     * @return {?}
     */
    reset() {
        this.provided.forEach((/**
         * @param {?} propertyDescriptor
         * @param {?} key
         * @return {?}
         */
        (propertyDescriptor, key) => {
            /** @type {?} */
            const value = this.component[key];
            Object.defineProperty(this.component, key, propertyDescriptor);
            this.component[key] = value;
        }));
        this.provided.clear();
        this.change$.next('');
        this.reset$.next();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        if (this.initialized) {
            this.reset();
            if (this.source)
                this.init();
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.initialized = true;
        this.ngOnChanges();
    }
}
ContextProviderComponent.decorators = [
    { type: Component, args: [{
                selector: 'context-provider',
                template: '<ng-content></ng-content>',
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None
            }] }
];
/** @nocollapse */
ContextProviderComponent.ctorParameters = () => [
    { type: ChangeDetectorRef, decorators: [{ type: Optional }, { type: SkipSelf }] }
];
ContextProviderComponent.propDecorators = {
    contextMap: [{ type: Input }],
    provide: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ContextConsumerComponent extends AbstractContextConsumer {
    /**
     * @param {?} providerComponent
     * @param {?} parent
     */
    constructor(providerComponent, parent) {
        super(providerComponent, parent);
    }
}
ContextConsumerComponent.decorators = [
    { type: Component, args: [{
                selector: 'context-consumer',
                template: '',
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None
            }] }
];
/** @nocollapse */
ContextConsumerComponent.ctorParameters = () => [
    { type: ContextProviderComponent, decorators: [{ type: Optional }, { type: SkipSelf }] },
    { type: ChangeDetectorRef, decorators: [{ type: Optional }, { type: SkipSelf }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ContextConsumerDirective extends AbstractContextConsumer {
    /**
     * @param {?} providerComponent
     * @param {?} host
     */
    constructor(providerComponent, host) {
        super(providerComponent, host);
        this.consume = '';
    }
}
ContextConsumerDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contextConsumer]',
            },] }
];
/** @nocollapse */
ContextConsumerDirective.ctorParameters = () => [
    { type: ContextProviderComponent, decorators: [{ type: Optional }, { type: SkipSelf }] },
    { type: ChangeDetectorRef, decorators: [{ type: Optional }, { type: Host }] }
];
ContextConsumerDirective.propDecorators = {
    consume: [{ type: Input, args: ['contextConsumer',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Context {
    constructor() {
        this.$implicit = {};
    }
}
class ContextDisposerDirective {
    /**
     * @param {?} tempRef
     * @param {?} vcRef
     * @param {?} provider
     */
    constructor(tempRef, vcRef, provider) {
        this.tempRef = tempRef;
        this.vcRef = vcRef;
        this.provider = provider;
        this.destroy$ = new Subject();
        this._dispose = '';
    }
    /**
     * @param {?} dispose
     * @return {?}
     */
    set dispose(dispose) {
        this._dispose = dispose || '';
    }
    /**
     * @return {?}
     */
    get dispose() {
        return this._dispose;
    }
    /**
     * @private
     * @return {?}
     */
    init() {
        /** @type {?} */
        const disposed = parseKeys(this.dispose);
        this.provider.reset$
            .pipe(takeUntil(this.destroy$))
            .subscribe((/**
         * @return {?}
         */
        () => this.ngOnChanges()));
        if (this.provider.provide.length)
            this.provider.change$
                .pipe(takeUntil(this.destroy$), startWith(...Array.from(this.provider.provided.keys())), filter((/**
             * @param {?} key
             * @return {?}
             */
            key => !!key)))
                .subscribe((/**
             * @param {?} providerKey
             * @return {?}
             */
            providerKey => this.syncProperties(disposed, providerKey)));
    }
    /**
     * @private
     * @return {?}
     */
    reset() {
        this.view = this.vcRef.createEmbeddedView(this.tempRef, new Context());
    }
    /**
     * @private
     * @param {?} disposed
     * @param {?} providerKey
     * @return {?}
     */
    syncProperties(disposed, providerKey) {
        /** @type {?} */
        const key = this.provider.contextMap[providerKey] || providerKey;
        if (disposed.length && disposed.indexOf(key) < 0)
            return;
        /** @type {?} */
        const value = this.provider.component[providerKey];
        this.view.context.$implicit[key] = value;
        this.view.context[key] = value;
        this.view.markForCheck();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.ngOnDestroy();
        this.reset();
        if (this.provider && this.tempRef && this.vcRef)
            this.init();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy$.next();
        if (this.view)
            this.vcRef.clear();
    }
}
ContextDisposerDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contextDisposer]',
            },] }
];
/** @nocollapse */
ContextDisposerDirective.ctorParameters = () => [
    { type: TemplateRef, decorators: [{ type: Optional }] },
    { type: ViewContainerRef, decorators: [{ type: Optional }] },
    { type: ContextProviderComponent, decorators: [{ type: Optional }, { type: SkipSelf }] }
];
ContextDisposerDirective.propDecorators = {
    dispose: [{ type: Input, args: ['contextDisposer',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxContextModule {
}
NgxContextModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ContextConsumerComponent,
                    ContextConsumerDirective,
                    ContextDisposerDirective,
                    ContextProviderComponent,
                ],
                exports: [
                    ContextConsumerComponent,
                    ContextConsumerDirective,
                    ContextDisposerDirective,
                    ContextProviderComponent,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxContextModule, ContextConsumerComponent, ContextConsumerDirective, ContextDisposerDirective, ContextProviderComponent, AbstractContextConsumer as ɵa };

//# sourceMappingURL=ngx-context.js.map